@@ -283,6 +283,8 @@ KAsync::Job<qint64> Pipeline::modifiedEntity(void const *command, size_t size)
             case Preprocessor::DropModification:
                 SinkTraceCtx(d->logCtx) << "Dropping modification";
                 return KAsync::error<qint64>(0);
+            case Preprocessor::NoAction:
+            case Preprocessor::DeleteEntity:
             default:
                 break;
         }
@@ -301,8 +303,8 @@ KAsync::Job<qint64> Pipeline::modifiedEntity(void const *command, size_t size)
                     SinkTraceCtx(d->logCtx) << "Move of " << current.identifier() << "was successfull";
                     if (isMove) {
                         flatbuffers::FlatBufferBuilder fbb;
-                        auto entityId = fbb.CreateString(current.identifier());
-                        auto type = fbb.CreateString(bufferType);
+                        auto entityId = fbb.CreateString(current.identifier().toStdString());
+                        auto type = fbb.CreateString(bufferType.toStdString());
                         auto location = Sink::Commands::CreateDeleteEntity(fbb, current.revision(), entityId, type, true);
                         Sink::Commands::FinishDeleteEntityBuffer(fbb, location);
                         const auto data = BufferUtils::extractBuffer(fbb);
@@ -448,8 +450,8 @@ void Preprocessor::createEntity(const Sink::ApplicationDomain::ApplicationDomain
     const auto entityBuffer = BufferUtils::extractBuffer(entityFbb);
 
     flatbuffers::FlatBufferBuilder fbb;
-    auto entityId = fbb.CreateString(entity.identifier());
-    auto type = fbb.CreateString(typeName);
+    auto entityId = fbb.CreateString(entity.identifier().toStdString());
+    auto type = fbb.CreateString(typeName.toStdString());
     auto delta = Sink::EntityBuffer::appendAsVector(fbb, entityBuffer.constData(), entityBuffer.size());
     auto location = Sink::Commands::CreateCreateEntity(fbb, entityId, type, delta);
     Sink::Commands::FinishCreateEntityBuffer(fbb, location);
