@@ -122,6 +122,18 @@ KAsync::Job<void> SINK_EXPORT synchronize(const Sink::SyncScope &query);
  */
 KAsync::Job<void> SINK_EXPORT removeDataFromDisk(const QByteArray &resourceIdentifier);
 
+/**
+ * Run upgrade jobs.
+ *
+ * Run this to upgrade your local database to a new version.
+ * Note that this may:
+ * * take a while
+ * * remove some/all of your local caches
+ *
+ * Note: The initial implementation simply calls removeDataFromDisk for all resources.
+ */
+KAsync::Job<void> SINK_EXPORT upgrade();
+
 template <class DomainType>
 KAsync::Job<DomainType> SINK_EXPORT fetchOne(const Sink::Query &query);
 
