@@ -31,6 +31,7 @@ namespace Imap {
 
 enum ErrorCode {
     NoError,
+    HostNotFoundError,
     CouldNotConnectError,
     SslHandshakeError
 };
@@ -60,6 +61,7 @@ namespace FolderFlags
     extern const char* Junk;
     extern const char* Flagged;
     extern const char* All;
+    extern const char* Drafts;
 }
 
 namespace Capabilities
@@ -78,6 +80,8 @@ struct Message {
     bool fullPayload;
 };
 
+bool flagsContain(const QByteArray &f, const QByteArrayList &flags);
+
 struct Folder {
     Folder() = default;
     Folder(const QString &path, const QString &ns, const QChar &separator, bool noselect_, bool subscribed_, const QByteArrayList &flags_)
@@ -114,6 +118,15 @@ struct Folder {
         return parentPath;
     }
 
+    Folder parentFolder() const
+    {
+        Folder parent;
+        parent.mPath = parentPath();
+        parent.mNamespace = mNamespace;
+        parent.mSeparator = mSeparator;
+        return parent;
+    }
+
     QString name() const
     {
         auto pathParts = mPath.split(mSeparator);
@@ -218,18 +231,18 @@ public:
                 return session;
             }
         }
-        return CachedSession{};
+        return {};
+    }
+
+    bool isEmpty() const
+    {
+        return mSessions.isEmpty();
     }
 private:
     QList<CachedSession> mSessions;
 };
 
 class ImapServerProxy {
-    KIMAP2::Session *mSession;
-    QStringList mCapabilities;
-    Namespaces mNamespaces;
-
-
 public:
     ImapServerProxy(const QString &serverUrl, int port, SessionCache *sessionCache = nullptr);
 
@@ -279,9 +292,14 @@ public:
     KAsync::Job<QVector<qint64>> fetchUids(const Folder &folder);
 
 private:
+    bool isGmail() const;
+
     QString getNamespace(const QString &name);
     QObject mGuard;
     SessionCache *mSessionCache;
+    KIMAP2::Session *mSession;
+    QStringList mCapabilities;
+    Namespaces mNamespaces;
 };
 
 }
